const chalk = require("chalk")
const helpers = require("./modules/helpers")
const R = require("rambdax")

const boxMethod = require("./modules/boxMethod")
const cancelMethod = require("./modules/cancelMethod")
const everyMethod = require("./modules/everyMethod")
const iconMethod = require("./modules/iconMethod")
const iconSetMethod = require("./modules/iconSetMethod")
const objMethod = require("./modules/objMethod")
const patternMethod = require("./modules/patternMethod")
const patternxMethod = require("./modules/patternxMethod")
const randomMethod = require("./modules/randomMethod")
const separatorMethod = require("./modules/separatorMethod")
const spinMethod = require("./modules/spinMethod")
const stopSpinMethod = require("./modules/stopSpinMethod")
const typeMethod = require("./modules/typeMethod")

const logMethod = (inputCollection, behaviourCollection) => {
  const rule = x => R.both(
    xInstance => R.type(xInstance) === "Object",
    xInstance => R.has("chalkRule")(xInstance)
  )(x)

  let defaultChalkRule

  const tag = helpers.getTag(behaviourCollection)
  const isBackMode = helpers.isMyMode("BACK", behaviourCollection)
  if (isBackMode === false && tag === false) {
    defaultChalkRule = chalk.hex(`#${ helpers.getChalkFront() }`)
  } else if (tag === false) {
    const { background, color } = helpers.getChalkBack()
    defaultChalkRule = chalk.hex(`#${ color }`).bgHex(`#${ background }`)
  } else if (helpers.patterns[ tag ] === undefined) {
    helpers.patterns[ tag ] = isBackMode === false ?
      helpers.getChalkFront() :
      helpers.getChalkBack()

    const settings = helpers.patterns[ tag ]
    defaultChalkRule = isBackMode === false ?
      chalk.hex(`#${ settings }`) :
      chalk.hex(`#${ settings.color }`).bgHex(`#${ settings.background }`)
  } else {
    const settings = helpers.patterns[ tag ]
    defaultChalkRule = isBackMode === false ?
      chalk.hex(`#${ settings }`) :
      chalk.hex(`#${ settings.color }`).bgHex(`#${ settings.background }`)
  }

  const internals = R.compose(
    R.ifElse(
      x => R.length(x) === 0,
      () => ({ chalkGenericRule : defaultChalkRule }),
      x => R.head(x).chalkRule
    ),
    R.filter(
      rule
    )
  )(inputCollection)

  const filteredInputCollection = R.filter(
    x => R.not(rule(x))
  )(inputCollection)

  if (R.type(internals) === "Object") {
    const objectsToLog = R.compose(
      R.filter(x => R.type(x) === "Object")
    )(filteredInputCollection)

    const inputCollectionToLog = R.compose(
      R.filter(x => R.type(x) !== "Object")
    )(filteredInputCollection)

    console.log(internals.chalkGenericRule(...inputCollectionToLog), ...objectsToLog)
  } else {
    console.log(internals, ...filteredInputCollection)
  }
}

const worker = (inputCollection, behaviourCollection) => {
  try {
    R.map(
      method => {
        const result = method(inputCollection, behaviourCollection)
        if (result === "EXIT") {
          throw "EXIT"
        }
        if (R.type(result) === "Array") {
          inputCollection = Array.from(result)
        }
      }
    )([
      randomMethod,
      everyMethod,
      cancelMethod,
      patternxMethod,
      patternMethod,
      typeMethod,
      boxMethod,
      separatorMethod,
      objMethod,
      spinMethod,
      stopSpinMethod,
      //above are the functions that might return 'EXIT'
      iconMethod,
      iconSetMethod,
      logMethod,
    ])
  } catch (err) {
    if (err !== "EXIT") {
      console.log(err)
    }
  }
}

function log (...args) {
  const inputCollection = R.ifElse(
    x => R.type(R.last(x)) === "String",
    x => x.length === 1 ? [ "" ] : R.init(x),
    R.identity
  )(args)

  const behaviourCollection = R.compose(
    R.map(R.toUpper),
    R.split("."),
    R.ifElse(
      x => R.type(R.last(x)) === "String",
      R.last,
      () => ""
    )
  )(args)

  worker(inputCollection, behaviourCollection)

  return true
}

module.exports = log
